<?php

/**
 * @file
 * Administrative forms for the Comemrce Be2Bill Payment Gatewayt module.
 */

/**
 * Implements hook_form().
 *
 * Form callback: allows the user to capture a prior authorization.
 */
function commerce_be2bill_capture_form($form, &$form_state, $order, $transaction) {
  $form_state['order'] = $order;
  $form_state['transaction'] = $transaction;

  // Load and store the payment method instance for this transaction.
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  $form_state['payment_method'] = $payment_method;

  $balance = commerce_payment_order_balance($order);

  if ($balance['amount'] > 0 && $balance['amount'] < $transaction->amount) {
    $default_amount = $balance['amount'];
  }
  else {
    $default_amount = $transaction->amount;
  }

  // Convert the price amount to a user friendly decimal value.
  $default_amount = commerce_currency_amount_to_decimal($default_amount, $transaction->currency_code);

  $description = implode('<br />', array(
    t('Authorization: @amount', array('@amount' => commerce_currency_format($transaction->amount, $transaction->currency_code))),
    t('Order balance: @balance', array('@balance' => commerce_currency_format($balance['amount'], $balance['currency_code']))),
  ));

  $form['amount'] = array(
    '#type' => 'textfield',
    '#title' => t('Capture amount'),
    '#description' => $description,
    '#default_value' => $default_amount,
    '#field_suffix' => check_plain($transaction->currency_code),
    '#size' => 16,
  );

  $form = confirm_form($form,
    t('What amount do you want to capture?'),
    'admin/commerce/orders/' . $order->order_id . '/payment',
    '',
    t('Capture'),
    t('Cancel'),
    'confirm'
  );

  return $form;
}

/**
 * Validate handler: ensure a valid amount is given.
 */
function commerce_be2bill_capture_form_validate($form, &$form_state) {
  $transaction = $form_state['transaction'];
  $amount = $form_state['values']['amount'];

  // Ensure a positive numeric amount has been entered for capture.
  if (!is_numeric($amount) || $amount <= 0) {
    form_set_error('amount', t('You must specify a positive numeric amount to capture.'));
  }

  // Ensure the amount is less than or equal to the authorization amount.
  if ($amount > commerce_currency_amount_to_decimal($transaction->amount, $transaction->currency_code)) {
    form_set_error('amount', t('You cannot capture more than you authorized through Be2Bill.'));
  }

  // If the authorization has expired, display an error message and redirect.
  if (time() - $transaction->created > 86400 * 7) {
    drupal_set_message(t('This authorization has passed its 7 day limit cannot be captured.'), 'error');
    drupal_goto('admin/commerce/orders/' . $form_state['order']->order_id . '/payment');
  }
}

/**
 * Submit handler: process a prior authorization capture.
 */
function commerce_be2bill_capture_form_submit($form, &$form_state) {
  $transaction = $form_state['transaction'];
  $amount = commerce_currency_decimal_to_amount($form_state['values']['amount'], $transaction->currency_code);
  $order = $form_state['order'];

  // Because the IPN request sent in synchronous to this form needs to load the
  // order at the same time we are obliged to release the lock to avoid a
  // timeout error.
  entity_get_controller('commerce_order')->resetCache(array($order->order_id));

  // Load and store the payment method instance for this transaction.
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  $settings = $payment_method['settings'];
  // Build the request to Be2Bill.
  $params = commerce_be2bill_request_capture($settings, $order, $transaction, $amount);
  $request = array(
    'method' => 'capture',
    'params' => $params,
  );
  $url = commerce_be2bill_server_url($settings['txn_mode'], 'server');
  $payload = http_build_query($request);
  $options = array(
    'method' => 'POST',
    'data' => $payload,
    'headers' => array('Content-Type' => 'application/x-www-form-urlencoded; charset=utf-8'),
  );
  $result = drupal_http_request($url, $options);
  $response = drupal_json_decode($result->data);
  // Update and save the transaction based on the response.
  $transaction->payload[REQUEST_TIME] = $response;
  // If we didn't get an approval response code...
  if ($response['EXECCODE'] != '0000' && $response['EXECCODE'] != '0001') {
    // Display an error message but leave the transaction pending.
    drupal_set_message(t('Prior authorization capture failed, so the transaction will remain in a pending status.'), 'error');
    drupal_set_message(check_plain($response['MESSAGE']), 'error');
  }
  else {
    drupal_set_message(t('Prior authorization captured successfully.'));

    // Update the transaction amount to the actual capture amount.
    $transaction->amount = $amount;

    // Set the remote and local status accordingly.
    $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
    $transaction->remote_status = $response['OPERATIONTYPE'];;
    $transaction->remote_id = $response['TRANSACTIONID'];

    // Append a capture indication to the result message.
    $transaction->message .= '<br />' . t('Captured: @date', array('@date' => format_date(REQUEST_TIME, 'short')));
  }

  commerce_payment_transaction_save($transaction);

  $form_state['redirect'] = 'admin/commerce/orders/' . $form_state['order']->order_id . '/payment';
}

/**
 * Implements hook_form().
 *
 * Form callback: allows the user to void a transaction.
 */
function commerce_be2bill_refund_void_form($form, &$form_state, $order, $transaction) {
  $form_state['order'] = $order;
  $form_state['transaction'] = $transaction;

  // Load and store the payment method instance for this transaction.
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  $form_state['payment_method'] = $payment_method;

  $balance = commerce_payment_order_balance($order);

  if ($balance['amount'] > 0 && $balance['amount'] < $transaction->amount) {
    $default_amount = $balance['amount'];
  }
  else {
    $default_amount = $transaction->amount;
  }

  // Convert the price amount to a user friendly decimal value.
  $default_amount = commerce_currency_amount_to_decimal($default_amount, $transaction->currency_code);

  $description = implode('<br />', array(
    t('Refund amount: @amount', array('@amount' => commerce_currency_format($transaction->amount, $transaction->currency_code))),
  ));

  $form['amount'] = array(
    '#type' => 'markup',
    '#title' => t('Refund amount'),
    '#markup' => $description,
  );

  $form = confirm_form($form,
    t('Are you sure you want to issue a refund?'),
    'admin/commerce/orders/' . $order->order_id . '/payment',
    '',
    t('Refund'),
    t('Cancel'),
    'confirm'
  );

  return $form;
}

/**
 * Validate handler: ensure a refund has not expired.
 */
function commerce_be2bill_refund_void_form_validate($form, &$form_state) {
  $transaction = $form_state['transaction'];

  // If the authorization has expired, display an error message and redirect.
  if (time() - $transaction->created > 86400 * 120) {
    drupal_set_message(t('This transaction has passed its 120 day limit and cannot be refunded.'), 'error');
    drupal_goto('admin/commerce/orders/' . $form_state['order']->order_id . '/payment');
  }
}

/**
 * Submit handler: process a refund of a prior transaction.
 */
function commerce_be2bill_refund_void_form_submit($form, &$form_state) {
  $transaction = $form_state['transaction'];
  $order = $form_state['order'];

  // Because the IPN request sent in synchronous to this form needs to load the
  // order at the same time we are obliged to release the lock to avoid a
  // timeout error.
  entity_get_controller('commerce_order')->resetCache(array($order->order_id));

  // Load and store the payment method instance for this transaction.
  $payment_method = commerce_payment_method_instance_load($transaction->instance_id);
  $settings = $payment_method['settings'];

  // Build the request to Be2Bill.
  $params = commerce_be2bill_request_refund_void($settings, $order, $transaction);
  $request = array(
    'method' => 'refund',
    'params' => $params,
  );

  $url = commerce_be2bill_server_url($settings['txn_mode'], 'server');
  $payload = http_build_query($request);
  $options = array(
    'method' => 'POST',
    'data' => $payload,
    'headers' => array('Content-Type' => 'application/x-www-form-urlencoded; charset=utf-8'),
  );
  $result = drupal_http_request($url, $options);
  $response = drupal_json_decode($result->data);
  // Update and save the transaction based on the response.
  $transaction->payload[REQUEST_TIME] = $response;
  // If we didn't get an approval response code...
  if ($response['EXECCODE'] != '0000' && $response['EXECCODE'] != '0001') {
    // Display an error message but leave the transaction pending.
    drupal_set_message(t('Refund failed, please see the message below.'), 'error');
    drupal_set_message(check_plain($response['MESSAGE']), 'error');
  }
  else {
    drupal_set_message(t('Refunded successfully.'));

    // Update the transaction amount.
    $transaction->amount = commerce_currency_decimal_to_amount(0, $transaction->currency_code);

    // Set the remote and local status accordingly.
    $transaction->status = COMMERCE_PAYMENT_STATUS_SUCCESS;
    $transaction->remote_status = $response['OPERATIONTYPE'];
    $transaction->remote_id = $response['TRANSACTIONID'];

    // Append a capture indication to the result message.
    $transaction->message .= '<br />' . t('Refunded: @date', array('@date' => format_date(REQUEST_TIME, 'short')));
  }

  commerce_payment_transaction_save($transaction);

  $form_state['redirect'] = 'admin/commerce/orders/' . $form_state['order']->order_id . '/payment';
}
